/* Allegro datafile object indexes, produced by grabber v4.1.13 (WIP), Unix */
/* Datafile: /home/kazzmir/archives/scratch/c++/raptor/rap/data/raptor.dat */
/* Date: Wed Jun  2 11:59:09 2004 */
/* Do not hand edit! */

#define BOSS_01                          0        /* BMP  */
#define BULLET_TWIRL                     1        /* BMP  */
#define CHAIN_LINK_CIRCLE                2        /* BMP  */
#define CHAIN_LINK_LINK                  3        /* BMP  */
#define CLOUD1                           4        /* BMP  */
#define CLOUD2                           5        /* BMP  */
#define COIN_1                           6        /* BMP  */
#define COIN_2                           7        /* BMP  */
#define COIN_3                           8        /* BMP  */
#define COIN_4                           9        /* BMP  */
#define COIN_5                           10       /* BMP  */
#define COIN_6                           11       /* BMP  */
#define COIN_7                           12       /* BMP  */
#define COIN_8                           13       /* BMP  */
#define COIN_9                           14       /* BMP  */
#define COIN_A_10                        15       /* BMP  */
#define COIN_A_11                        16       /* BMP  */
#define COIN_A_12                        17       /* BMP  */
#define COIN_A_13                        18       /* BMP  */
#define COIN_A_14                        19       /* BMP  */
#define COIN_A_15                        20       /* BMP  */
#define COIN_A_16                        21       /* BMP  */
#define COIN_A_17                        22       /* BMP  */
#define COIN_A_18                        23       /* BMP  */
#define COIN_A_19                        24       /* BMP  */
#define COIN_A_20                        25       /* BMP  */
#define EXPLODE_1                        26       /* BMP  */
#define EXPLODE_2                        27       /* BMP  */
#define EXPLODE_3                        28       /* BMP  */
#define EXPLODE_4                        29       /* BMP  */
#define EXPLODE_5                        30       /* BMP  */
#define EXPLODE_6                        31       /* BMP  */
#define EXPLODE_7                        32       /* BMP  */
#define EXPLODE_8                        33       /* BMP  */
#define EXPLODE_9                        34       /* BMP  */
#define EXPLODE_A_10                     35       /* BMP  */
#define EXPLODE_A_11                     36       /* BMP  */
#define EXPLODE_A_12                     37       /* BMP  */
#define EXPLODE_A_13                     38       /* BMP  */
#define EXPLODE_A_14                     39       /* BMP  */
#define EXPLODE_A_15                     40       /* BMP  */
#define EXPLODE_A_16                     41       /* BMP  */
#define EXPLODE_A_17                     42       /* BMP  */
#define EXPLODE_A_18                     43       /* BMP  */
#define EXPLODE_A_19                     44       /* BMP  */
#define EXPLODE_A_20                     45       /* BMP  */
#define EXPLODE_A_21                     46       /* BMP  */
#define EXPLODE_A_22                     47       /* BMP  */
#define EXPLODE_A_23                     48       /* BMP  */
#define EXPLODE_A_24                     49       /* BMP  */
#define EXPLODE_A_25                     50       /* BMP  */
#define EXPLODE_A_26                     51       /* BMP  */
#define EXPLODE_A_27                     52       /* BMP  */
#define EXPLODE_A_28                     53       /* BMP  */
#define EXPLODE_A_29                     54       /* BMP  */
#define EXPLODE_A_30                     55       /* BMP  */
#define GR_EXPLODE_01                    56       /* BMP  */
#define GR_EXPLODE_02                    57       /* BMP  */
#define GR_EXPLODE_03                    58       /* BMP  */
#define GR_EXPLODE_04                    59       /* BMP  */
#define GR_EXPLODE_05                    60       /* BMP  */
#define GR_EXPLODE_06                    61       /* BMP  */
#define GR_EXPLODE_07                    62       /* BMP  */
#define GR_EXPLODE_08                    63       /* BMP  */
#define GR_EXPLODE_09                    64       /* BMP  */
#define GR_EXPLODE_10                    65       /* BMP  */
#define GR_EXPLODE_11                    66       /* BMP  */
#define GR_EXPLODE_12                    67       /* BMP  */
#define GR_EXPLODE_13                    68       /* BMP  */
#define GR_EXPLODE_14                    69       /* BMP  */
#define GR_EXPLODE_15                    70       /* BMP  */
#define GR_EXPLODE_16                    71       /* BMP  */
#define GR_EXPLODE_17                    72       /* BMP  */
#define GR_EXPLODE_18                    73       /* BMP  */
#define GR_EXPLODE_19                    74       /* BMP  */
#define GR_EXPLODE_20                    75       /* BMP  */
#define GR_EXPLODE_21                    76       /* BMP  */
#define GR_EXPLODE_22                    77       /* BMP  */
#define GR_EXPLODE_23                    78       /* BMP  */
#define GR_EXPLODE_24                    79       /* BMP  */
#define GR_EXPLODE_25                    80       /* BMP  */
#define GR_EXPLODE_26                    81       /* BMP  */
#define GR_EXPLODE_27                    82       /* BMP  */
#define GR_EXPLODE_28                    83       /* BMP  */
#define GR_EXPLODE_29                    84       /* BMP  */
#define HEALTH_001                       85       /* BMP  */
#define HEALTH_002                       86       /* BMP  */
#define HEALTH_003                       87       /* BMP  */
#define HEALTH_004                       88       /* BMP  */
#define HEALTH_005                       89       /* BMP  */
#define HEALTH_006                       90       /* BMP  */
#define HEALTH_007                       91       /* BMP  */
#define HEALTH_008                       92       /* BMP  */
#define HEALTH_009                       93       /* BMP  */
#define HEALTH_010                       94       /* BMP  */
#define HEALTH_011                       95       /* BMP  */
#define HEALTH_012                       96       /* BMP  */
#define HEALTH_013                       97       /* BMP  */
#define HEALTH_014                       98       /* BMP  */
#define HEALTH_015                       99       /* BMP  */
#define HEALTH_016                       100      /* BMP  */
#define HEALTH_017                       101      /* BMP  */
#define HEALTH_018                       102      /* BMP  */
#define HEALTH_019                       103      /* BMP  */
#define HEALTH_020                       104      /* BMP  */
#define HEALTH_021                       105      /* BMP  */
#define HELICOPTER_1                     106      /* BMP  */
#define HELICOPTER_2                     107      /* BMP  */
#define HELICOPTER_3                     108      /* BMP  */
#define HELICOPTER_4                     109      /* BMP  */
#define HELICOPTER_5                     110      /* BMP  */
#define HELICOPTER_6                     111      /* BMP  */
#define HULL_1_1                         112      /* BMP  */
#define HULL_1_2                         113      /* BMP  */
#define HULL_1_3                         114      /* BMP  */
#define HULL_1_4                         115      /* BMP  */
#define HULL_1_5                         116      /* BMP  */
#define HULL_1_6                         117      /* BMP  */
#define HULL_1_7                         118      /* BMP  */
#define HULL_1_8                         119      /* BMP  */
#define HULL_1_9                         120      /* BMP  */
#define HULL_2_1                         121      /* BMP  */
#define HULL_2_2                         122      /* BMP  */
#define HULL_2_3                         123      /* BMP  */
#define HULL_2_4                         124      /* BMP  */
#define HULL_2_5                         125      /* BMP  */
#define HULL_2_6                         126      /* BMP  */
#define HULL_2_7                         127      /* BMP  */
#define HULL_2_8                         128      /* BMP  */
#define HULL_2_9                         129      /* BMP  */
#define HULL_3_1                         130      /* BMP  */
#define HULL_3_2                         131      /* BMP  */
#define HULL_3_3                         132      /* BMP  */
#define HULL_3_4                         133      /* BMP  */
#define HULL_3_5                         134      /* BMP  */
#define HULL_3_6                         135      /* BMP  */
#define HULL_3_7                         136      /* BMP  */
#define HULL_3_8                         137      /* BMP  */
#define HULL_3_9                         138      /* BMP  */
#define IRON_000                         139      /* BMP  */
#define IRON_001                         140      /* BMP  */
#define IRON_002                         141      /* BMP  */
#define IRON_003                         142      /* BMP  */
#define IRON_004                         143      /* BMP  */
#define IRON_005                         144      /* BMP  */
#define IRON_006                         145      /* BMP  */
#define IRON_007                         146      /* BMP  */
#define IRON_008                         147      /* BMP  */
#define IRON_009                         148      /* BMP  */
#define IRON_010                         149      /* BMP  */
#define IRON_011                         150      /* BMP  */
#define IRON_012                         151      /* BMP  */
#define METEOR_1                         152      /* BMP  */
#define METEOR_2                         153      /* BMP  */
#define METEOR_3                         154      /* BMP  */
#define METEOR_4                         155      /* BMP  */
#define METEOR_5                         156      /* BMP  */
#define METEOR_6                         157      /* BMP  */
#define METEOR_7                         158      /* BMP  */
#define METEOR_8                         159      /* BMP  */
#define METEOR_9                         160      /* BMP  */
#define METEOR_9_10                      161      /* BMP  */
#define METEOR_9_11                      162      /* BMP  */
#define METEOR_9_12                      163      /* BMP  */
#define METEOR_9_13                      164      /* BMP  */
#define METEOR_9_14                      165      /* BMP  */
#define METEOR_9_15                      166      /* BMP  */
#define METEOR_9_16                      167      /* BMP  */
#define METEOR_9_17                      168      /* BMP  */
#define METEOR_9_18                      169      /* BMP  */
#define METEOR_9_19                      170      /* BMP  */
#define METEOR_9_20                      171      /* BMP  */
#define POWERUP_1                        172      /* BMP  */
#define POWERUP_2                        173      /* BMP  */
#define POWERUP_3                        174      /* BMP  */
#define POWERUP_4                        175      /* BMP  */
#define POWERUP_5                        176      /* BMP  */
#define POWERUP_6                        177      /* BMP  */
#define POWERUP_7                        178      /* BMP  */
#define POWERUP_8                        179      /* BMP  */
#define POWERUP_9                        180      /* BMP  */
#define POWERUP_A_10                     181      /* BMP  */
#define POWERUP_A_11                     182      /* BMP  */
#define POWERUP_A_12                     183      /* BMP  */
#define POWERUP_A_13                     184      /* BMP  */
#define POWERUP_A_14                     185      /* BMP  */
#define SHIP0_FIRST                      186      /* BMP  */
#define SHIP_001                         187      /* BMP  */
#define SHIP_002                         188      /* BMP  */
#define SHIP_003                         189      /* BMP  */
#define SHIP_004                         190      /* BMP  */
#define SHIP_005                         191      /* BMP  */
#define SHIP_006                         192      /* BMP  */
#define SHIP_007                         193      /* BMP  */
#define SHIP_008                         194      /* BMP  */
#define SHIP_009                         195      /* BMP  */
#define SHIP_010                         196      /* BMP  */
#define SHIP_LAST                        197      /* BMP  */
#define WEAPON_BOMB_1                    198      /* BMP  */
#define WEAPON_BOMB_2                    199      /* BMP  */
#define WEAPON_BOMB_3                    200      /* BMP  */
#define WEAPON_BOMB_4                    201      /* BMP  */
#define WEAPON_BOMB_5                    202      /* BMP  */
#define WEAPON_BOMB_6                    203      /* BMP  */
#define WEAPON_BOMB_7                    204      /* BMP  */
#define WEAPONUP_001                     205      /* BMP  */
#define WEAPONUP_002                     206      /* BMP  */
#define WEAPONUP_003                     207      /* BMP  */
#define WEAPONUP_004                     208      /* BMP  */
#define WEAPONUP_005                     209      /* BMP  */
#define WEAPONUP_006                     210      /* BMP  */
#define WEAPONUP_007                     211      /* BMP  */
#define WEAPONUP_008                     212      /* BMP  */
#define WEAPONUP_009                     213      /* BMP  */
#define WEAPONUP_010                     214      /* BMP  */
#define WEAPONUP_011                     215      /* BMP  */
#define WEAPONUP_012                     216      /* BMP  */
#define WEAPONUP_013                     217      /* BMP  */
#define WEAPONUP_014                     218      /* BMP  */
#define WEAPONUP_015                     219      /* BMP  */
#define WEAPONUP_016                     220      /* BMP  */
#define WEAPONUP_017                     221      /* BMP  */
#define WEAPONUP_018                     222      /* BMP  */
#define WEAPONUP_019                     223      /* BMP  */
#define WEAPONUP_020                     224      /* BMP  */
#define Z_MAX_DRAW                       225      /* BMP  */

