/* Allegro datafile object indexes, produced by grabber v4.1.13 (WIP), Unix */
/* Datafile: /home/kazzmir/archives/scratch/c++/raptor/rap/data/sound.dat */
/* Date: Sat May  8 12:27:08 2004 */
/* Do not hand edit! */

#define INTRO_MENU_SELECT                0        /* SAMP */
#define MENU_BUTTON                      1        /* SAMP */
#define SND_CODE_RED                     2        /* SAMP */
#define SND_EXPLODE_1                    3        /* SAMP */
#define SND_EXPLODE_2                    4        /* SAMP */
#define SND_FIGHT                        5        /* SAMP */
#define SND_GUN_1                        6        /* SAMP */
#define SND_HEALTH                       7        /* SAMP */
#define SND_HULL_COLLIDE                 8        /* SAMP */
#define SND_METAL                        9        /* SAMP */
#define SND_POWER                        10       /* SAMP */
#define SND_SUSTAIN                      11       /* SAMP */

