/* Allegro datafile object indexes, produced by grabber v4.1.13 (WIP), Unix */
/* Datafile: /home/kazzmir/archives/scratch/c++/jezzball/PCX_fonts/fonts.dat */
/* Date: Wed Mar 31 14:12:06 2004 */
/* Do not hand edit! */

#define ANDALEMONO_PCX                   0        /* FONT */
#define BROADWAY_PCX                     1        /* FONT */
#define CALIGRAPH_PCX                    2        /* FONT */
#define CATANEO_PCX                      3        /* FONT */
#define CENTURYGOTHIC_PCX                4        /* FONT */
#define CORSIVA_PCX                      5        /* FONT */
#define FIXEDSYS_PCX                     6        /* FONT */
#define HIGHLIGHT_PCX                    7        /* FONT */
#define IRONMAIDEN_PCX                   8        /* FONT */
#define JOHNHANDY_PCX                    9        /* FONT */
#define JOKERMAN_PCX                     10       /* FONT */
#define KUNSTLER_PCX                     11       /* FONT */
#define LABAMBA_PCX                      12       /* FONT */
#define LCD_PCX                          13       /* FONT */
#define MEKANIK_PCX                      14       /* FONT */
#define METALLORD_PCX                    15       /* FONT */
#define MISTER_PCX                       16       /* FONT */
#define ODESSA_PCX                       17       /* FONT */
#define OLDDREAD_PCX                     18       /* FONT */
#define ONESTROKE_PCX                    19       /* FONT */
#define ORANGE_PCX                       20       /* FONT */
#define PARKAVENUE_PCX                   21       /* FONT */
#define PUMP_PCX                         22       /* FONT */
#define QUIXLEY_PCX                      23       /* FONT */
#define RAGE_PCX                         24       /* FONT */
#define RAUCH_PCX                        25       /* FONT */
#define SCRUFF_PCX                       26       /* FONT */
#define SMUDGER_PCX                      27       /* FONT */
#define SQUARE_PCX                       28       /* FONT */
#define STACCATO_PCX                     29       /* FONT */
#define TIRANTI_PCX                      30       /* FONT */
#define VICTORAIN_PCX                    31       /* FONT */
#define WOFACHABCER_PCX                  32       /* FONT */
#define WOFALDE_PCX                      33       /* FONT */
#define WOFAMBROS_PCX                    34       /* FONT */
#define WOFAMERUNI_PCX                   35       /* FONT */
#define WOFANDRO_PCX                     36       /* FONT */
#define WOFARGENT_PCX                    37       /* FONT */
#define WOFARIZONA_PCX                   38       /* FONT */
#define WOFARROW_PCX                     39       /* FONT */
#define WOFARTE_PCX                      40       /* FONT */
#define WOFBAIDER_PCX                    41       /* FONT */
#define WOFBARIUM_PCX                    42       /* FONT */
#define WOFBASTIA_PCX                    43       /* FONT */
#define WOFBATS_PCX                      44       /* FONT */
#define WOFBILIA_PCX                     45       /* FONT */
#define WOFBLADE_PCX                     46       /* FONT */
#define WOFBLIXA_PCX                     47       /* FONT */
#define WOFBONIX_PCX                     48       /* FONT */
#define WOFBORDER_PCX                    49       /* FONT */
#define WOFBOROLD_PCX                    50       /* FONT */
#define WOFBORSCH_PCX                    51       /* FONT */
#define WOFBURO_PCX                      52       /* FONT */
#define WOFCANDI_PCX                     53       /* FONT */
#define WOFCHIRO_PCX                     54       /* FONT */
#define WOFCLIPS_PCX                     55       /* FONT */
#define WOFCLOCK_PCX                     56       /* FONT */
#define WOFCLOCKER_PCX                   57       /* FONT */
#define WOFCOOL_PCX                      58       /* FONT */
#define WOFDELLI_PCX                     59       /* FONT */
#define WOFDRAKER_PCX                    60       /* FONT */
#define WOFEDUK_PCX                      61       /* FONT */
#define WOFELECTRO_PCX                   62       /* FONT */
#define WOFFIRNIS_PCX                    63       /* FONT */
#define WOFFITIO_PCX                     64       /* FONT */
#define WOFFLOW_PCX                      65       /* FONT */
#define WOFFUTUR_PCX                     66       /* FONT */
#define WOFGINEWRA_PCX                   67       /* FONT */
#define WOFHANDS_PCX                     68       /* FONT */
#define WOFHARDSH_PCX                    69       /* FONT */
#define WOFINTONA_PCX                    70       /* FONT */
#define WOFJOKER_PCX                     71       /* FONT */
#define WOFNEWART_PCX                    72       /* FONT */
#define ZLAST                            73       /* BMP  */

