#ifndef _strings_h
#define _strings_h

#include <string>

int numDig( int q );
std::string int2str( int z );
std::string int2normal( int z );
// string d2normal( double whole, int precise );
int str2int( char * convert );
double str2d( char * convert );
int length( char * q );
char upcase( char u );
void upstring( char * who );

#endif
